# -*- coding: utf-8 -*-

import os
import sys
import locale
import gtk
import traceback
from subprocess import Popen
from threading import Thread

HAS_WNCK = True
try:
    import wnck
except:
    HAS_WNCK = False

DBUS = True
try:
    import dbus
    import dbus._version
    from dbus.mainloop.glib import DBusGMainLoop
    assert dbus.version >= (0, 80, 0)
except ImportError, AssertionError:
    #~ sys.stderr.write("This program needs dbus-python 0.80.0 or later\nExiting\n")
    #~ sys.exit(1)
    DBUS = False

ICON_THEME = gtk.icon_theme_get_default()

## ---------

# Only for debugging : False / True
DEBUG = 0

VERSION = 0.3
RC = 4
NAME = 'ADeskBar'

## ---------


cat_name = {'Accessories':('Utility','Accessories'),
            'Games':('Game'),
            'Graphics':('Graphics'),
            'Network':('Internet', 'Network'),
            'Office':('Office'),
            'Other':('Other'),
            'Development':('Development'),
            'Settings':('Settings', 'X-XFCE'),
            'Multimedia':('AudioVideo', 'Audio', 'Video', 'Multimedia'),
            'System':('System', 'Administration'),
            }

cat_icon = {'Accessories':'applications-utilities',
            'Games':'applications-games',
            'Graphics':'applications-graphics',
            'Network':'applications-internet',
            'Office':'applications-office',
            'Other':'applications-other',
            'Development':'applications-development',
            'Settings':'preferences-desktop',
            'Multimedia':'applications-multimedia',
            'System':'applications-system',
            }

def init_dbus():
    if DBUS:
        print "DBUS OK ..."
        try:
            bus = dbus.SessionBus(mainloop = DBusGMainLoop())
        except dbus.exceptions.DBusException:
            sys.stderr.write("Couldn't connect to dbus.\nExiting\n")
            return False
            #~ sys.exit(1)
        if bus.request_name("com.adcomp.adeskbar") != dbus.bus.REQUEST_NAME_REPLY_PRIMARY_OWNER:
            sys.stderr.write("Only one DBUS instance is allowed\n")
            #~ sys.exit(1)
        return True
    else:
        return False

# Find the Name / Command / Icon from .desktop file
def info_desktop(file, term_cmd=None):
    cmd, icon, name, category = None, None, None, 'Other'
    terminal = False
    try:
        cfile = open(file,"r")

        for line in cfile:
            if 'NoDisplay=true' in line: # or 'OnlyShowIn=KDE' in line:
                icon, category = None, None
                break
            elif 'Terminal=true' in line:
                terminal = True
            elif '=' in line:
                words = line.split('=')

                if words[0] == 'Name':
                    name = words[1].replace('\n','')
                    # print 'Name =', name
                elif words[0] == 'Icon':
                    icon = words[1].replace('\n','')
                    
                    # sometimes , Icon is fullpath or icon name + ext
                    # why ? So here we remove ext if exist
                    if not icon[0] == '/':
                        icon = icon.split('.',1)[0]
                elif words[0] == 'Exec':
                    cmd = words[1].replace('\n','')
                    cmd = cmd.split('%')[0]
                    # print 'Exec :', cmd
                elif words[0] == 'Categories':
                    tab = words[1].replace('\n','')
                    tab = tab.split(';')
                    #~ category = '***************************'

                    #~ if not 'X-XFCE' in tab:

                    for cat in tab:
                        found = False
                        for c_name in cat_name:
                            if cat in cat_name[c_name]:
                                category = c_name
                                found = True
                                break
                        if found:
                            break

        # if command is 'console only', launch it with terminal ..
        if terminal:
            if term_cmd:
                cmd = "%s -e %s" % (term_cmd, cmd)
            else:
                cmd = "x-terminal-emulator -e %s" % cmd

        cfile.close()

    except:
        print("# Error : parsing %s" % file)
        pass

    return (cmd, icon, name, category)

# Find the best icon with highest resolution for the launcher
def find_icon(icon):
    foundiconfile=None

    if icon == '' or icon == None:
        return foundiconfile

    if icon[0] == '/':
        return icon

    iconbase=('','gnome','hicolor')
    iconpath='/usr/share/icons'
    sizelist =('', 'scalable', '256x256', '128x128', '64x64', '48x48', '32x32', '24x24')
    categorylist=('actions', 'apps',"devices", 'categories','filesystems', 'places', 'status', '')
    extensionlist = ('png', 'svg', 'xpm')

    iconimagelist=[]

    for extension in extensionlist:
        if (icon.find('.'+extension) != -1):
            icon = icon.replace('.'+extension,'')

    for size in sizelist:
        for extension in extensionlist:
            for category in categorylist:
                for iconbasecat in iconbase:
                    iconfile = iconpath+"/"+iconbasecat+'/'+size+'/'+category+'/'+icon+'.'+extension
                    iconimagelist.append(iconfile)

    for extension in extensionlist:
        iconfile = '/usr/share/pixmaps/'+icon+'.'+extension
        iconimagelist.append(iconfile)

    for extension in extensionlist:
        iconfile = '/usr/share/app-install/icons/'+icon+'.'+extension
        iconimagelist.append(iconfile)

    # Seek if the files in pre-generated list exists.. first match is the best
    # return it
    for iconimage in iconimagelist:
        if os.path.exists(iconimage):
            return iconimage

    return foundiconfile

def parse_user_menu():
    config = {}
    cfg_file = home = os.getenv('HOME') + '/.config/adesk-menu/config'

    if os.access(cfg_file, os.F_OK|os.R_OK):
        print('found user config ..')
        f = open(cfg_file,'r')

        for line in f:
            if line == '\n' or line.startswith('#'):
                continue
            elif line.startswith('@'):
                cat = line[1:]
                cat = cat.strip('\n')
                cat = cat.strip(' ')
                config[cat] = []
                cat_index = cat
            else:
                line = line.strip('\n')
                line = line.strip(' ')
                config[cat_index].append(line.split('##'))
        f.close()
    #~ print config
    return config

def parse_desktop_dir(desktop_dir='/usr/share/applications/', term_cmd=None):
    config = {}
    logINFO('scan %s and parse .desktop file ..' % desktop_dir)
    
    desktop_dir = []
    home = os.environ['HOME']
    
    for dir in ('/usr/share/applications/', 
                "%s/.local/share/applications/" % home,
                "/usr/local/share/applications/",
                "/usr/share/applications/kde4/",
                "/usr/local/share/applications/kde4/"):
        if os.path.exists(dir):
            desktop_dir.append(dir)


    for dir in desktop_dir:
        listdir = os.listdir(dir)

        for i in listdir:
            if '.desktop' in i:
                desktop_file = dir + i
                (cmd, icon, name, category) = info_desktop(desktop_file, term_cmd)

                if category:
                    if not config.has_key(category):
                        #~ print "@ add :", category
                        config[category] = []
                    if icon:
                        #~ icon_path = find_icon(icon)
                        #~ config[category].append((cmd, icon_path, name, desktop_file))
                        config[category].append((cmd, icon, name, desktop_file))
                    else:
                        config[category].append((cmd, 'images/def_icon.png', name, desktop_file))
                else:
                    ## No category ?
                    pass
                
    return config

def logINFO(msg, from_mod=""):
    if DEBUG:
        print "[ADesk.%s] %s" % (from_mod, msg)

def launch_command(cmd):
    if cmd != ' ' and cmd != None and len(cmd)!=0:
        os.chdir(os.environ['HOME'])
        logINFO("Exec. | %s |" % cmd)
        ## FIXME! replace os.system with subprocess.Popen
        #~ os.system('sh -c "%s" &' % cmd)
        p = Popen(cmd, shell=True).pid
        #~ sts = os.waitpid(p.pid, 0)[1]
        realpath = os.path.dirname(os.path.realpath( __file__ ))
        os.chdir(realpath + '/..')

def hex2rgb(color_hex):
    ## convert Hex color to RGB
    hexcolor = color_hex.strip()
    hexcolor = hexcolor[1:]
    if len(hexcolor) != 6:
        logINFO('Invalid hex color, use #RRGGBB format.')
        return (0.0, 0.0, 0.0)
    r, g, b = hexcolor[:2], hexcolor[2:4], hexcolor[4:]
    r, g, b = [int(n, 16) for n in (r, g, b)]
    return (float(r)/256, float(g)/256, float(b)/256)

def pixbuf_from_file(file, width=None, height=None):
    pixbuf = None

    if file != None or file == '':
        if os.path.isfile(file):
            try:
                if not width and not height:
                    pixbuf = gtk.gdk.pixbuf_new_from_file(file)
                else:
                    width, height = int(width), int(height)
                    pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(file, width, height)
            except:
                logINFO("is a image ? => %s" % file)
                pixbuf = None
        else:
            logINFO("is a image ? => %s" % file)

    return pixbuf

def ask_question(msg='Ok .. ?'):
    """  """
    message = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO, msg)
    resp = message.run()
    message.destroy()

    if resp == gtk.RESPONSE_YES:
        return True
    else:
        return False

def show_msg(msg=' .. '):
    """  """
    message = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, msg)
    resp = message.run()
    message.destroy()

def image_button(label, image, size):
    bt = gtk.Button()
    bt.set_relief(gtk.RELIEF_NONE)
    bt.set_border_width(0)
    bt.set_focus_on_click(False)
    #~ bt.set_property('can-focus', False)

    if image == None:
        bt_image = gtk.Image()
        pixbuf = pixbuf_from_file('images/def_icon.png', size, size)
        bt_image.set_from_pixbuf(pixbuf)
    elif type(image) == gtk.Image:
        bt_image = image
    elif type(image) == gtk.gdk.Pixbuf:
        bt_image = gtk.Image()
        bt_image.set_from_pixbuf(image)
    else:
        bt_image = gtk.Image()
        try:
            pixbuf = pixbuf_from_file(image, size, size)
            bt_image.set_from_pixbuf(pixbuf)
        except:
            pixbuf = pixbuf_from_file('images/def_icon.png', size, size)
            bt_image.set_from_pixbuf(pixbuf)
        
    bt_image.show()
    bt_image.set_size_request(size, size)

    bt_label = gtk.Label()
    bt_label.set_use_markup(True)
    bt_label.set_markup(label)
    bt_label.set_alignment(0.2, 1)
    align = gtk.Alignment(1, 0, 0.5, 0.5)
    align.show()
    align.add(bt_label)

    box = gtk.HBox(False, 6)
    box.set_border_width(2)
    box.show()
    box.pack_start(bt_image, False, False)
    box.pack_start(align, False, False)
    bt.add(box)
    bt.show_all()
    return bt

class Plugin(gtk.EventBox):
    def __init__(self, app, settings):
        gtk.EventBox.__init__(self)
        self.set_visible_window(False)
        self.app = app
        self.cfg = app.cfg
        self.settings = settings
        self.index = None

        self.pixbuf = None
        self.pixbuf_zoom = None
        self.current_icon = None

        self.can_zoom = False
        self.can_show_icon = True

        self.is_visible = False
        self.tooltip = ''
        self.has_tooltip = True
        self.show()

    def set_tooltip(self, tooltip):
        self.tooltip = tooltip
        self.app.tooltip.set_tooltip(tooltip)
        self.app.tooltip.reposition(self)

    def get_tooltip(self, tooltip):
        return self.tooltip

    def set_icon(self, path_icon, is_separator=False):
        #~ print 'set_icon :', path_icon
        if path_icon == '' or self.current_icon == path_icon:
            return

        w = h = self.cfg['icon_size']
        self.pixbuf = None

        if is_separator:
            if self.cfg['position'] == 'bottom' or self.cfg['position'] == 'top':
                w = int(w/4)
            else:
                h = int(h/4)

        if not path_icon:
            self.pixbuf = pixbuf_from_file('images/def_icon.png', w, h)
            self.pixbuf_zoom = pixbuf_from_file('images/def_icon.png', self.app.zoom_size, self.app.zoom_size)
            self.current_icon = 'images/def_icon.png'
            
        ## real path .. load from file
        elif path_icon[0] == '/' or ( len(path_icon) > 7 and path_icon[:7] == 'images/'):
            self.pixbuf = pixbuf_from_file(path_icon, w, h)
            self.pixbuf_zoom = pixbuf_from_file(path_icon, self.app.zoom_size, self.app.zoom_size)
            self.current_icon = path_icon
            
        ## load from icon theme
        elif ICON_THEME.has_icon(path_icon):
            self.pixbuf = ICON_THEME.load_icon(path_icon, self.cfg['icon_size'], gtk.ICON_LOOKUP_USE_BUILTIN)
            self.pixbuf_zoom = ICON_THEME.load_icon(path_icon, int(self.app.zoom_size), gtk.ICON_LOOKUP_USE_BUILTIN)
            self.current_icon = path_icon


        ## check if pixbuf is ok
        if not self.pixbuf:
            self.pixbuf = pixbuf_from_file('images/def_icon.png', w, h)
            self.pixbuf_zoom = pixbuf_from_file('images/def_icon.png', self.app.zoom_size, self.app.zoom_size)
            self.current_icon = 'images/def_icon.png'

        if self.app.init_flag:
            self.app.update()

    def draw(self, cr):
        pass

    def resize(self):
        pass

    def onClick(self, widget, event):
        pass

    def init_position(self):

        x, y, width, height = self.get_allocation()
        x0, y0 = x, y

        size_press = int(self.cfg['icon_size'] * 0.9)

        zoom_size = int(self.cfg['icon_size'] * self.cfg['zoom_factor'])
        zoom_size_press = int(self.cfg['icon_size'] * self.cfg['zoom_factor']*0.9)

        zoom_offset1 =  int((zoom_size - self.cfg['icon_size'])/2.0)
        zoom_offset2 = int((self.cfg['icon_size'] + self.cfg['padding']/2.0) -  zoom_size)

        offset_click = (self.cfg['icon_size'] - self.cfg['icon_size']* 0.9)/2.0

        offset_click1 = int((self.cfg['icon_size'] * self.cfg['zoom_factor'] *0.9 - self.cfg['icon_size'])/2.0)
        offset_click2 = int((self.cfg['icon_size'] + self.cfg['padding']/2.0) -  (self.cfg['icon_size'] * self.cfg['zoom_factor'] *0.9))

        if self.cfg['position'] == "top":
            x1 = x - zoom_offset1
            y1 = self.cfg['padding']/2

            y0 = self.cfg['padding']/2
            x0 = x - offset_click1

        elif self.cfg['position'] == "bottom":
            y1 = y + zoom_offset2  ## zoom_offset2 est négatif !! => pas de soustraction

            y1 = self.app.bar_height - ( self.cfg['padding']/2.0 + self.cfg['icon_size'] * self.cfg['zoom_factor'] )
            x1 = x - zoom_offset1

            y0 = self.app.bar_height - ( self.cfg['padding']/2.0 + self.cfg['icon_size'] * self.cfg['zoom_factor']*0.9 )
            x0 = x - offset_click1

        elif self.cfg['position'] == "left":
            y1 = y - zoom_offset1
            x1 = self.cfg['padding']/2

            x0 = self.cfg['padding']/2 # self.app.bar_width - ( self.cfg['padding']/2.0 + self.cfg['icon_size'] * self.cfg['zoom_factor']*0.9 )
            y0 = y - offset_click1

        elif self.cfg['position'] == "right":
            y1 = y - zoom_offset1
            x1 = x +  zoom_offset2 ## 2*zoom_offset1 + self.cfg['padding']/2

            x0 = self.app.bar_width - ( self.cfg['padding']/2.0 + self.cfg['icon_size'] * self.cfg['zoom_factor']*0.9 )
            y0 = y - offset_click1

        self.x = x
        self.y = y
        self.x0 = x0
        self.y0 = y0
        self.x1 = x1
        self.y1 = y1
        self.offset_click = offset_click
        self.offset_click1 = offset_click1
        self.offset_click2 = offset_click2
        self.zoom_offset1 = zoom_offset1
        self.zoom_offset2 = zoom_offset2
        self.zoom_size = zoom_size
        self.zoom_size_press = zoom_size_press

class Menu():
    def __init__(self):
        self.menu = None
        self.is_ready = False

class MakeMenu(Thread):
    def __init__ (self, menu, notebook, callback_action, terminal=None):
        Thread.__init__(self)
        self.menu = menu
        self.notebook = notebook
        self.callback_action = callback_action
        self.terminal = terminal

    def run(self):

        if self.menu.menu == None:
            menu = parse_desktop_dir(term_cmd=self.terminal)
        else:
            menu = self.menu.menu

        cat_tmp = []

        for categoty in menu:
            cat_tmp.append(categoty)
        cat_tmp.sort()

        self.cat_visible = True

        for category in cat_tmp:
            #~ print category

            bBox = gtk.VBox()
            bBox.show()
            bBox.set_spacing(1)
            bBox.set_border_width(1)

            scrolled = gtk.ScrolledWindow()
            scrolled.show()
            scrolled.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
            scrolled.add_with_viewport(bBox)

            tab_box = gtk.HBox(False, 4)
            tab_label = gtk.Label(category)
            tab_icon = gtk.Image()
            tab_icon.set_from_icon_name(cat_icon[category], 24)

            tab_box.pack_start(tab_icon, False)
            tab_box.pack_start(tab_label, False)

            # needed, otherwise even calling show_all on the notebook won't
            # make the hbox contents appear.
            tab_box.show_all()

            gtk.gdk.threads_enter()
            self.notebook.append_page(scrolled, tab_box)
            gtk.gdk.threads_leave()

            for application in menu[category]:
                if application[1]:
                    if application[1][0] == '/':
                        button = image_button(application[2], application[1], 24)
                    else:
                        icon = gtk.Image()
                        icon.set_from_icon_name(application[1], 24)
                        button = image_button(application[2], icon, 24)

                #~ button.set_tooltip_text(application[3])
                button.set_name(application[3])
                button.connect("button-release-event", self.callback_action, application)
                bBox.pack_start(button, False, False)

        self.menu.is_ready = True
        self.menu.menu = menu
        #~ print 'menu is_ready ...'

class Wnck:
    def __init__(self, bar):
        self.bar = bar
        self.current_state = False

        # silently ignore x errors
        gtk.gdk.error_trap_push()

        # init screen and hook into WnckSceen's events
        self.screen = wnck.screen_get_default()
        self.screen.force_update()
        self.screen.connect("window_opened", self.window_opened)
        self.screen.connect("window_closed", self.window_closed)
        self.screen.connect("active-workspace-changed", self.workspace_changed)

        windows = self.screen.get_windows()
        for window in windows:
            self.window_opened(self.screen, window)

        self.is_maximized(windows)

    # define event-handlers
    def workspace_changed(self, screen, workspace):
        self.is_maximized()

    def window_opened(self, screen, window):
        #~ print 'new window open ..'
        window.connect("geometry-changed", self.geometry_changed)
        window.connect("state_changed", self.state_changed)
        if window.is_maximized():
            #~ print '- is_maximized'
            self.current_state = True
            self.bar.check_window_state()

    def geometry_changed(self, window):
        self.is_maximized()

    def state_changed(self, window, changed, new_state):
        self.is_maximized()

    def window_closed(self, screen, window):
        self.is_maximized()

    def is_maximized(self, windows=None):
        current_wp = self.screen.get_active_workspace()

        if windows == None:
            windows = self.screen.get_windows()

        _state = False

        for window in windows:
            if current_wp == window.get_workspace():
                if window.is_maximized() and not window.is_minimized():
                    _state = True
                    break

        if not self.current_state == _state:
            self.current_state = _state
            self.bar.check_window_state()

